@@ -1,4 +1,3 @@
-#include <net/if.h>
 #include <errno.h>
 #include <string.h>
 #include <stdbool.h>
@@ -17,7 +16,9 @@
 			"control:  show control frames\n"\
 			"otherbss: show frames from other BSSes\n"\
 			"cook:     use cooked mode\n"\
-			"active:   use active mode (ACK incoming unicast packets)"
+			"active:   use active mode (ACK incoming unicast packets)\n"\
+			"mumimo-groupid <GROUP_ID>: use MUMIMO according to a group id\n"\
+			"mumimo-follow-mac <MAC_ADDRESS>: use MUMIMO according to a MAC address"
 
 SECTION(interface);
 
@@ -31,6 +32,55 @@ static char *mntr_flags[NL80211_MNTR_FLAG_MAX + 1] = {
 	"active",
 };
 
+static int parse_mumimo_options(int *_argc, char ***_argv, struct nl_msg *msg)
+{
+	uint8_t mumimo_group[VHT_MUMIMO_GROUP_LEN];
+	unsigned char mac_addr[ETH_ALEN];
+	char **argv = *_argv;
+	int argc = *_argc;
+	int i;
+	unsigned int val;
+
+	if (strcmp(*argv, "mumimo-groupid") == 0) {
+		argc--;
+		argv++;
+		if (!argc || strlen(*argv) != VHT_MUMIMO_GROUP_LEN*2) {
+			fprintf(stderr, "Invalid groupID: %s\n", *argv);
+			return 1;
+		}
+
+		for (i = 0; i < VHT_MUMIMO_GROUP_LEN; i++) {
+			if (sscanf((*argv) + i*2, "%2x", &val) != 1) {
+				fprintf(stderr, "Failed reading groupID\n");
+				return 1;
+			}
+			mumimo_group[i] = val;
+		}
+
+		NLA_PUT(msg,
+			NL80211_ATTR_MU_MIMO_GROUP_DATA,
+			VHT_MUMIMO_GROUP_LEN,
+			mumimo_group);
+		argc--;
+		argv++;
+	} else if (strcmp(*argv, "mumimo-follow-mac") == 0) {
+		argc--;
+		argv++;
+		if (!argc || mac_addr_a2n(mac_addr, *argv)) {
+			fprintf(stderr, "Invalid MAC address\n");
+			return 1;
+		}
+		NLA_PUT(msg, NL80211_ATTR_MU_MIMO_FOLLOW_MAC_ADDR,
+			ETH_ALEN, mac_addr);
+		argc--;
+		argv++;
+	}
+ nla_put_failure:
+	*_argc = argc;
+	*_argv = argv;
+	return 0;
+}
+
 static int parse_mntr_flags(int *_argc, char ***_argv,
 			    struct nl_msg *msg)
 {
@@ -46,6 +96,15 @@ static int parse_mntr_flags(int *_argc, char ***_argv,
 
 	while (argc) {
 		int ok = 0;
+
+		/* parse MU-MIMO options */
+		err = parse_mumimo_options(&argc, &argv, msg);
+		if (err)
+			goto out;
+		else if (!argc)
+			break;
+
+		/* parse monitor flags */
 		for (flag = __NL80211_MNTR_FLAG_INVALID;
 		     flag <= NL80211_MNTR_FLAG_MAX; flag++) {
 			if (strcmp(*argv, mntr_flags[flag]) == 0) {
@@ -144,6 +203,9 @@ static int get_if_type(int *argc, char ***argv, enum nl80211_iftype *type,
 	} else if (strcmp(tpstr, "__p2pgo") == 0) {
 		*type = NL80211_IFTYPE_P2P_GO;
 		return 0;
+	} else if (strcmp(tpstr, "__nan") == 0) {
+		*type = NL80211_IFTYPE_NAN;
+		return 0;
 	}
 
 	fprintf(stderr, "invalid interface type %s\n", tpstr);
@@ -296,6 +358,10 @@ char *channel_width_name(enum nl80211_chan_width width)
 		return "80+80 MHz";
 	case NL80211_CHAN_WIDTH_160:
 		return "160 MHz";
+	case NL80211_CHAN_WIDTH_5:
+		return "5 MHz";
+	case NL80211_CHAN_WIDTH_10:
+		return "10 MHz";
 	default:
 		return "unknown";
 	}
@@ -368,6 +434,25 @@ static int print_iface_handler(struct nl_msg *msg, void *arg)
 		printf("\n");
 	}
 
+	if (tb_msg[NL80211_ATTR_WIPHY_TX_POWER_LEVEL]) {
+		uint32_t txp = nla_get_u32(tb_msg[NL80211_ATTR_WIPHY_TX_POWER_LEVEL]);
+
+		printf("%s\ttxpower %d.%.2d dBm\n",
+		       indent, txp / 100, txp % 100);
+	}
+
+	if (tb_msg[NL80211_ATTR_TXQ_STATS]) {
+		char buf[150];
+		parse_txq_stats(buf, sizeof(buf), tb_msg[NL80211_ATTR_TXQ_STATS], 1, -1, indent);
+		printf("%s\tmulticast TXQ:%s\n", indent, buf);
+	}
+
+	if (tb_msg[NL80211_ATTR_4ADDR]) {
+		uint8_t use_4addr = nla_get_u8(tb_msg[NL80211_ATTR_4ADDR]);
+		if (use_4addr)
+			printf("%s\t4addr: on\n", indent);
+	}
+
 	return NL_SKIP;
 }
 
@@ -395,6 +480,8 @@ static int handle_interface_set(struct nl80211_state *state,
 	switch (parse_mntr_flags(&argc, &argv, msg)) {
 	case 0:
 		return 0;
+	case 1:
+		return 1;
 	case -ENOMEM:
 		fprintf(stderr, "failed to allocate flags\n");
 		return 2;
@@ -552,10 +639,8 @@ static int set_mcast_rate(struct nl80211_state *state,
 	float rate;
 	char *end;
 
-	if (argc != 1) {
-		printf("Invalid parameters!\n");
-		return 2;
-	}
+	if (argc != 1)
+		return 1;
 
 	rate = strtod(argv[0], &end);
 	if (*end != '\0')
@@ -571,3 +656,74 @@ nla_put_failure:
 COMMAND(set, mcast_rate, "<rate in Mbps>",
 	NL80211_CMD_SET_MCAST_RATE, 0, CIB_NETDEV, set_mcast_rate,
 	"Set the multicast bitrate.");
+
+
+static int handle_chanfreq(struct nl80211_state *state, struct nl_msg *msg,
+			   bool chan, int argc, char **argv,
+			   enum id_input id)
+{
+	struct chandef chandef;
+	int res;
+	int parsed;
+	char *end;
+
+	res = parse_freqchan(&chandef, chan, argc, argv, &parsed);
+	if (res)
+		return res;
+
+	argc -= parsed;
+	argv += parsed;
+
+	while (argc) {
+		unsigned int beacons = 10;
+
+		if (strcmp(argv[0], "beacons") == 0) {
+			if (argc < 2)
+				return 1;
+
+			beacons = strtol(argv[1], &end, 10);
+			if (*end)
+				return 1;
+
+			argc -= 2;
+			argv += 2;
+
+			NLA_PUT_U32(msg, NL80211_ATTR_CH_SWITCH_COUNT, beacons);
+		} else if (strcmp(argv[0], "block-tx") == 0) {
+			argc -= 1;
+			argv += 1;
+
+			NLA_PUT_FLAG(msg, NL80211_ATTR_CH_SWITCH_BLOCK_TX);
+		} else {
+			return 1;
+		}
+	}
+
+	return put_chandef(msg, &chandef);
+
+ nla_put_failure:
+	return -ENOBUFS;
+}
+
+static int handle_freq(struct nl80211_state *state, struct nl_msg *msg,
+		       int argc, char **argv,
+		       enum id_input id)
+{
+	return handle_chanfreq(state, msg, false, argc, argv, id);
+}
+
+static int handle_chan(struct nl80211_state *state, struct nl_msg *msg,
+		       int argc, char **argv,
+		       enum id_input id)
+{
+	return handle_chanfreq(state, msg, true, argc, argv, id);
+}
+
+SECTION(switch);
+COMMAND(switch, freq,
+	"<freq> [NOHT|HT20|HT40+|HT40-|5MHz|10MHz|80MHz] [beacons <count>] [block-tx]\n"
+	"<control freq> [5|10|20|40|80|80+80|160] [<center1_freq> [<center2_freq>]] [beacons <count>] [block-tx]",
+	NL80211_CMD_CHANNEL_SWITCH, 0, CIB_NETDEV, handle_freq,
+	"Switch the operating channel by sending a channel switch announcement (CSA).");
+COMMAND(switch, channel, "<channel> [NOHT|HT20|HT40+|HT40-|5MHz|10MHz|80MHz] [beacons <count>] [block-tx]",
+	NL80211_CMD_CHANNEL_SWITCH, 0, CIB_NETDEV, handle_chan, NULL);
